@@ -22,6 +22,7 @@ setGeneric("corpus", function(.Object, ...) standardGeneric("corpus"))
 #' @importFrom RcppCWB cqp_list_corpora corpus_data_dir corpus_registry_dir
 #'   corpus_info_file corpus_full_name
 #' @importFrom fs path path_expand
+#' @importFrom cli cli_bullets
 setMethod("corpus", "character", function(
     .Object, registry_dir,
     server = NULL, restricted
@@ -63,10 +64,17 @@ setMethod("corpus", "character", function(
     c_regdir <- path(corpus_registry_dir(.Object))
     if (missing(registry_dir)){
       if (length(c_regdir) > 1L){
-        stop(
-          "Cannot initialize corpus object - ",
-          "corpus defined by two different registry files."
-        )
+        c_regdir <- path(unique(path(c_regdir)))
+        if (length(c_regdir) > 1L){
+          cli_alert_warning(
+            "corpus loaded multiple times with following registries:"
+          )
+          cli_bullets(setNames(c_regdir, rep("*", times = length(c_regdir))))
+          cli_alert_info(
+            "using {c_regdir[1]}"
+          )
+        }
+        registry_dir <- c_regdir[1]
       } else if (is.na(c_regdir)){
         stop(
           "Cannot initialize corpus object - ",
@@ -77,16 +85,9 @@ setMethod("corpus", "character", function(
       }
     } else {
       registry_dir <- path(path_expand(registry_dir))
-      if (is.na(c_regdir)){
-        stop("Cannot locate corpus with registry provided.")
-      } else if (length(c_regdir) > 1L){
-        if (!registry_dir %in% c_regdir){
-          stop("Cannot locate corpus with registry provided.")
-        }
-      } else if (registry_dir != c_regdir){
+      if (!registry_dir %in% c_regdir){
         stop("Cannot locate corpus with registry provided.")
       }
-      
     }
     
     properties <- sapply(
@@ -236,8 +237,8 @@ setMethod("corpus", "missing", function(){
 #'   includes undefined s-attributes, a warning is issued and the return value
 #'   is `NULL`.
 #' @rdname subset
-#' @name subset
-#' @aliases subset,corpus-method
+#' @name subset-method
+#' @aliases subset subset,corpus-method
 #' @seealso The methods applicable for the `subcorpus` object resulting from
 #'   subsetting a corpus or subcorpus are described in the documentation of the
 #'   `\link{subcorpus-class}`. Note that the `subset`-method can also be applied
@@ -279,21 +280,22 @@ setMethod("corpus", "missing", function(){
 #'   c("Angela Dorothea Merkel", "Volker Kauder", "Ronald Pofalla"),
 #'   function(who) subset(gparl, speaker == !!who)
 #' )
-#' @param x A \code{corpus} or \code{subcorpus} object. A corpus may also
-#'   specified by a length-one \code{character} vector.
+#' @param x A `corpus` or `subcorpus` object. A corpus may also specified by a
+#'   length-one `character` vector.
 #' @param ... An expression that will be used to create a subcorpus from
 #'   s-attributes.
-#' @param subset A \code{logical} expression indicating elements or rows to
-#'   keep. The expression may be unevaluated (using \code{quote} or
-#'   \code{bquote}).
+#' @param verbose A `logical` value, whether to show messages.
+#' @param subset A `logical` expression indicating elements or rows to
+#'   keep. The expression may be unevaluated (using `quote()` or
+#'   `bquote()`).
 #' @importFrom data.table setindexv setDT
 #' @importFrom rlang enquo eval_tidy is_quosure
-#' @param regex A \code{logical} value. If \code{TRUE}, values for s-attributes
+#' @param regex A `logical` value. If `TRUE`, values for s-attributes
 #'   defined using the three dots (...) are interpreted as regular expressions
-#'   and passed into a \code{grep} call for subsetting a table with the regions
-#'   and values of structural attributes. If \code{FALSE} (the default), values
+#'   and passed into a `grep` call for subsetting a table with the regions
+#'   and values of structural attributes. If `FALSE` (the default), values
 #'   for s-attributes must match exactly.
-setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
+setMethod("subset", "corpus", function(x, subset, regex = FALSE, verbose = FALSE, ...){
   stopifnot(is.logical(regex))
   s_attr <- character()
 
@@ -318,14 +320,15 @@ setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
       is_call <- FALSE
     }
 
-    s_attr_expr <- s_attributes(expr, corpus = x) # get s_attributes present in the expression
+    # get s_attributes present in the expression
+    s_attr_expr <- s_attributes(expr, corpus = x) 
     s_attr <- c(s_attr, s_attr_expr)
   }
 
   dots <- list(...)
   if (length(dots) > 0L){
     if (!all(names(dots) %in% s_attributes(x))){
-      stop("Aborting - at least one of the s-attributes provided as an argument is not available.")
+      stop("Aborting - at least one s-attributes required is not available.")
     }
     s_attr_dots <- names(dots)
     s_attr <- c(s_attr, s_attr_dots)
@@ -342,9 +345,9 @@ setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
     return(NULL)
   }
 
-  # Reading the binary file with the ranges for the whole corpus is faster than using
-  # the RcppCWB functionality. The assumption here is that the XML is flat, i.e. no need
-  # to read in seperate rng files.
+  # Reading the binary file with the ranges for the whole corpus is faster than
+  # using the RcppCWB functionality. The assumption here is that the XML is
+  # flat, i.e. no need to read in seperate rng files.
   if (!s_attr[1] %in% s_attributes(x)){
     warning(sprintf("structural attribute '%s' not defined", s_attr[1]))
     return(NULL)
@@ -352,7 +355,13 @@ setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
     
   rng_file <- fs::path(x@data_dir, paste(s_attr[1], "rng", sep = "."))
   rng_size <- file.info(rng_file)[["size"]]
-  rng <- readBin(rng_file, what = integer(), size = 4L, n = rng_size / 4L, endian = "big")
+  rng <- readBin(
+    rng_file,
+    what = integer(),
+    size = 4L,
+    n = rng_size / 4L,
+    endian = "big"
+  )
   dt <- data.table(
     struc = 0L:((length(rng) / 2L) - 1L),
     cpos_left = rng[seq.int(from = 1L, to = length(rng), by = 2L)],
@@ -371,7 +380,15 @@ setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
       avs = fs::path(x@data_dir, paste(s_attr[i], "avs", sep = ".")),
       avx = fs::path(x@data_dir, paste(s_attr[i], "avx", sep = "."))
     )
-    if (all(file.exists(unlist(files)))){
+    if (
+      all(c(
+          all(file.exists(unlist(files))),
+          (names(s_attr)[i] != "integer")
+      ))
+      ){
+      if (verbose) cli_alert_info(
+        "preparing s-attribute {.val {s_attr[i]}} (using decoded values)"
+      )
       sizes <- lapply(files, function(file) file.info(file)[["size"]])
       
       avx <- readBin(
@@ -392,7 +409,20 @@ setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
       
       dt[, (s_attr[i]) := avs[match(avx_matrix[, 2], unique(avx_matrix[, 2]))] ]
     } else {
-      cli_alert_info("s_attribute {.val {s_attr[i]}} does not have values")
+      if (verbose){
+        cli_alert_info(
+          "preparing s-attribute {.val {s_attr[i]}} (undecoded struc numbers)"
+        )
+      }
+      
+      attr_size <- cl_attribute_size(
+        corpus = x@corpus,
+        attribute = s_attr[i],
+        attribute_type = "s",
+        registry = x@registry_dir
+        
+      )
+      dt[, (s_attr[i]) := 0L:(attr_size - 1L)]
     }
   }
   
@@ -429,7 +459,7 @@ setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
 
   # And assemble the subcorpus object that is returned.
   if (nrow(dt) == 0L){
-    warning("No matching regions found for the s-attributes provided: Returning NULL object")
+    warning("No matching regions found for the s-attributes provided (returning NULL)")
     return(NULL)
   }
   
@@ -442,7 +472,7 @@ setMethod("subset", "corpus", function(x, subset, regex = FALSE, ...){
   y@cpos <- as.matrix(dt[, c("cpos_left", "cpos_right")])
   dimnames(y@cpos) <- NULL
   y@strucs = dt[["struc"]]
-  y@s_attribute_strucs <- s_attr[length(s_attr)]
+  y@s_attribute_strucs <- unname(s_attr[length(s_attr)])
   y@s_attributes <- lapply(setNames(s_attr, s_attr), function(s) unique(dt[[s]]))
   y@xml <- x@xml
   y@name <- ""
@@ -459,8 +489,8 @@ setMethod("subset", "character", function(x, ...){
 
 
 #' @rdname subset
-#' @importFrom RcppCWB s_attr_regions
-setMethod("subset", "subcorpus", function(x, subset, ...){
+#' @importFrom RcppCWB s_attr_regions region_matrix_to_struc_matrix
+setMethod("subset", "subcorpus", function(x, subset, verbose = FALSE, ...){
   expr <- enquo(subset)
   evaluated <- tryCatch(
     eval_tidy(expr),
@@ -473,6 +503,7 @@ setMethod("subset", "subcorpus", function(x, subset, ...){
 
   # get s_attributes present in the expression
   s_attr <- s_attributes(expr, corpus = x) 
+  
   dt <- data.table(
     struc = x@strucs,
     cpos_left = x@cpos[, 1],
@@ -497,17 +528,18 @@ setMethod("subset", "subcorpus", function(x, subset, ...){
       setNames(s_attr, s_attr),
       function(s)
         s_attr_regions(
-          corpus = x@corpus, registry = x@registry_dir,
+          corpus = x@corpus,
+          registry = x@registry_dir,
           data_dir = x@data_dir,
           s_attr = s
         )
     )
-
+    
     if (length(s_attr) > 1L){
       # When subsetting on more than one s-attribute at a time, it is required
       # that these are on the same level.
-      if (length(unique(s_attr_sizes[-1L])) == 1L){
-        stop("structural attributes for subsetting need to have identical size - not true")
+      if (length(unique(s_attr_sizes[-1L])) != 1L){
+        stop("s-attributes required to have identical size - not true")
       }
       
       # The second check is whether regions of s-attributes for subsetting are
@@ -518,10 +550,12 @@ setMethod("subset", "subcorpus", function(x, subset, ...){
       
     }
     
-    for (s in s_attr){
+    for (i in 1L:length(s_attr)){
       strucs <- cl_cpos2struc(
-        corpus = x@corpus, registry = x@registry_dir,
-        s_attribute = s, cpos = x@cpos[,1]
+        corpus = x@corpus,
+        registry = x@registry_dir,
+        s_attribute = s_attr[i],
+        cpos = x@cpos[,1]
       )
       
       if (any(is.na(strucs)) || any(strucs < 0L)){
@@ -529,15 +563,30 @@ setMethod("subset", "subcorpus", function(x, subset, ...){
         break
       }
 
-      r <- matrix(regions[[s]][strucs + 1L,], ncol = 2)
+      r <- matrix(regions[[s_attr[i]]][strucs + 1L,], ncol = 2L)
       if (all(r[,1] <= x@cpos[,1]) && all(r[,2] >= x@cpos[,2])){
         descendant <- FALSE
-        str <- cl_struc2str(
-          corpus = x@corpus, registry = x@registry_dir,
-          s_attribute = s, struc = strucs
-        )
-        Encoding(str) <- x@encoding
-        dt[, (s_attr) := str]
+        if (names(s_attr)[i] == "integer"){
+          if (verbose) cli_alert_info(
+            "preparing s-attribute {.val {s_attr[i]}} (undecoded struc numbers)"
+          )
+          dt[, (s_attr[i]) := strucs]
+        } else if (s_attr_has_values(s_attribute = s_attr[i], x = x)){
+          if (verbose) cli_alert_info(
+            "preparing s-attribute {.val {s_attr[i]}} (using decoded values)"
+          )
+          str <- cl_struc2str(
+            corpus = x@corpus,
+            registry = x@registry_dir,
+            s_attribute = s_attr[i],
+            struc = strucs
+          )
+          Encoding(str) <- x@encoding
+          dt[, (s_attr[i]) := str]
+        } else {
+          stop("s-attribute does not have values")
+        }
+        
       } else {
         descendant <- TRUE
         break
@@ -545,12 +594,19 @@ setMethod("subset", "subcorpus", function(x, subset, ...){
     }
     
     if (descendant){
-      # Slower by necessity
-      strucs <- cl_cpos2struc(
-        x@corpus, registry = x@registry_dir,
-        s_attribute = s_attr[1], cpos = ranges_to_cpos(x@cpos)
+      struc_matrix <- region_matrix_to_struc_matrix(
+        corpus = x@corpus,
+        s_attribute = s_attr[1],
+        region_matrix = x@cpos,
+        registry = x@registry_dir
       )
-      strucs <- unique(strucs[strucs >= 0L])
+      na_rows <- apply(struc_matrix, 1, function(row) any(is.na(row)))
+      if (any(na_rows)) struc_matrix <- struc_matrix[-na_rows,]
+      
+      nomatch <- which(struc_matrix[,1] < 0L)
+      if (length(nomatch) > 0L) struc_matrix <- struc_matrix[-nomatch,]
+      
+      strucs <- ranges_to_cpos(struc_matrix)
       ranges <- get_region_matrix(
         x@corpus, registry = x@registry_dir,
         s_attribute = s_attr[1], strucs = strucs
@@ -563,44 +619,88 @@ setMethod("subset", "subcorpus", function(x, subset, ...){
       
       # this is a somewhat dirty assumption that s_attr will not have values
       if (is.call(quo_get_expr(expr))){
-        str <- cl_struc2str(
-          corpus = x@corpus, registry = x@registry_dir,
-          s_attribute = s_attr[1], struc = strucs
-        )
-        Encoding(str) <- x@encoding
-        dt[, (s_attr[1]) := str]
+        if (names(s_attr)[1] == "integer"){
+          if (verbose){
+            cli_alert_info(
+              "preparing s-attribute {.val {s_attr[1]}} (undecoded struc numbers)"
+            )
+          }
+          dt[, (s_attr[1]) := strucs]
+        } else if (s_attr_has_values(s_attribute = s_attr[i], x = x)){
+          if (verbose)
+            cli_alert_info(
+              "preparing s-attribute {.val {s_attr[1]}} (using decoded values)"
+            )
+          str <- cl_struc2str(
+            corpus = x@corpus,
+            registry = x@registry_dir,
+            s_attribute = s_attr[1],
+            struc = strucs
+          )
+          Encoding(str) <- x@encoding
+          dt[, (s_attr[1]) := str]
+        }
+      } else {
+        stop("s-attribute does not have values")
       }
       
       if (length(s_attr) > 1L){
         for (s in s_attr[-1]){
-          str <- cl_struc2str(
-            corpus = x@corpus, registry = x@registry_dir,
-            s_attribute = s, struc = strucs
-          )
-          Encoding(str) <- x@encoding
-          dt[, (s) := str]
+          if (names(s_attr)[which(s_attr == s)] == "integer"){
+            if (verbose){
+              cli_alert_info(
+                "preparing s-attribute {.val {s}} (undecoded struc numbers)"
+              )
+            }
+            dt[, (s) := strucs]
+          } else if (s_attr_has_values(s_attribute = s, x = x)){
+            if (verbose)
+              cli_alert_info(
+                "preparing s-attribute {.val {s}} (using decoded values)"
+              )
+            str <- cl_struc2str(
+              corpus = x@corpus,
+              registry = x@registry_dir,
+              s_attribute = s,
+              struc = strucs
+            )
+            Encoding(str) <- x@encoding
+            dt[, (s) := str]
+          } else {
+            stop("s-attribute does not have values")
+          }
         }
       }
+      x@s_attribute_strucs = unname(s_attr[length(s_attr)])
     }
-    
+
   } else {
     for (s in s_attr){
-      str <- cl_struc2str(
-        corpus = x@corpus, registry = x@registry_dir,
-        s_attribute = s, struc = dt[["struc"]]
-      )
-      Encoding(str) <- x@encoding
-      dt[, (s) := str]
+      if (names(s_attr)[which(s_attr == s)] == "integer"){
+        dt[, (s) := dt[["struc"]]]
+      } else if (s_attr_has_values(s_attribute = s, x = x)){
+        str <- cl_struc2str(
+          corpus = x@corpus,
+          registry = x@registry_dir,
+          s_attribute = s,
+          struc = dt[["struc"]]
+        )
+        Encoding(str) <- x@encoding
+        dt[, (s) := str]
+      } else {
+        stop("s-attribute does not have values")
+      }
     }
-    
+    x@s_attribute_strucs = unname(s_attr[length(s_attr)])
   }
   
-  x@s_attribute_strucs = s_attr[length(s_attr)]
-  
   if (is.call(quo_get_expr(expr))){
     setindexv(dt, cols = s_attr)
     success <- try({dt_min <- dt[eval_tidy(expr, data = dt)]})
-    if (is(success, "try-error")) return(NULL)
+    if (is(success, "try-error")){
+      cli_alert_info("using expression for subsetting fails")
+      return(NULL)
+    }
     x@cpos <- as.matrix(dt_min[, c("cpos_left", "cpos_right")])
     x@strucs <- dt_min[["struc"]]
     x@s_attributes <- c(
@@ -623,10 +723,15 @@ setMethod("subset", "subcorpus", function(x, subset, ...){
 #' @details The `show()`-method will show basic information on the
 #'   `corpus` object.
 setMethod("show", "corpus", function(object){
-  message(sprintf("<< %s >>", class(object)))
+  message(sprintf("<<%s>>", class(object)))
   message(sprintf("%-12s", "corpus:"), object@corpus)
   message(sprintf("%-12s", "encoding:"), object@encoding)
-  message(sprintf("%-12s", "type:"), if (length(object@type) > 0) object@type else "[undefined]")
+  message(
+    sprintf("%-12s", "type:"), if (length(object@type) > 0) object@type else "[undefined]"
+  )
+  message(
+    sprintf("%-12s", "template:"), if (is.na(object@template)) "no" else "yes"
+  )
   message(sprintf("%-12s", "size:"), size(object))
 })
 
