@@ -1,3 +1,5 @@
+#' @name decode-method
+#' @aliases decode
 #' @rdname decode
 setGeneric("decode", function(.Object, ...) standardGeneric("decode"))
 
@@ -12,13 +14,28 @@ setAs(from = "corpus", to = "Annotation", def = function(from){
   if (!requireNamespace(package = "NLP", quietly = TRUE))
     stop("Package 'NLP' required but not available")
 
-  word <- get_token_stream(from@cpos, corpus = from@corpus, p_attribute = "word", encoding = from@encoding)
-  pos <- get_token_stream(from@cpos, corpus = from@corpus, p_attribute = "pos", encoding = from@encoding)
-  whitespace_after <- c(ifelse(pos %in% c("$.", "$,"), FALSE, TRUE)[2L:length(pos)], FALSE)
-  word_with_whitespace <- paste(word, ifelse(whitespace_after, " ", ""), sep = "")
-  s <- paste(word_with_whitespace, collapse = "")
+  word <- get_token_stream(
+    from@cpos,
+    corpus = from@corpus,
+    p_attribute = "word",
+    encoding = from@encoding
+  )
+  
+  if (!"pos" %in% p_attributes(from)) stop("p-attribute not available")
+  pos <- get_token_stream(
+    from@cpos,
+    corpus = from@corpus,
+    p_attribute = "pos",
+    encoding = from@encoding
+  )
+  ws_after <- c(ifelse(pos %in% c("$.", "$,"), FALSE, TRUE)[-1], FALSE)
+  word_with_ws <- paste(word, ifelse(ws_after, " ", ""), sep = "")
+  s <- paste(word_with_ws, collapse = "")
   word_length <- sapply(word, nchar)
-  left_offset <- c(1L, (cumsum(sapply(word_with_whitespace, nchar)) + 1L)[1L:(length(word) - 1L)] )
+  left_offset <- c(
+    1L,
+    (cumsum(sapply(word_with_ws, nchar)) + 1L)[1L:(length(word) - 1L)]
+  )
   names(left_offset) <- word
   right_offset <- left_offset + word_length - 1L
   names(right_offset) <- word
@@ -33,7 +50,11 @@ setAs(from = "corpus", to = "Annotation", def = function(from){
   right_offset <- left_offset + word_length
   names(right_offset) <- word # repeats 
   m <- matrix(data = c(left_offset, right_offset), ncol = 2, byrow = FALSE)
-  f <- cut(x = 1L:length(pos), breaks = unique(c(1L, grep("\\$\\.", pos), length(pos))), include.lowest = TRUE)
+  f <- cut(
+    x = 1L:length(pos),
+    breaks = unique(c(1L, grep("\\$\\.", pos), length(pos))), 
+    include.lowest = TRUE
+  )
   chunks <- split(x = m, f = f)
   sentence_left <- sapply(chunks, min)
   sentence_right <- sapply(chunks, max) - 1L
@@ -47,139 +68,190 @@ setAs(from = "corpus", to = "Annotation", def = function(from){
 })
 
 
-setAs(from = "corpus", to = "AnnotatedPlainTextDocument", def = function(from){
-  
-  # Implemented only for class 'corpus', 'subcorpus'-class will inherit from it.
+#' @importFrom RcppCWB region_matrix_to_struc_matrix
+as.AnnotatedPlainTextDocument <- function(x, p_attributes = NULL, s_attributes = NULL, mw = NULL, stoplist = NULL, verbose = TRUE){
+  # is required to be a subcorpus object
   
+  stopifnot(
+    inherits(x, "subcorpus"),
+    length(mw) == length(unique(mw))
+  )
+
   if (!requireNamespace(package = "NLP", quietly = TRUE))
     stop("Package 'NLP' required but not available.")
   
-  message("... decode p-attributes")
-  p_attrs <- p_attributes(from)
-  ts <- decode(from, p_attribute = p_attrs, s_attributes = character())
-
-  message("... generate plain text string")
-  whitespace_after <- if ("pos" %in% p_attrs){
-    # still assumes that STSS is used
-    c(ifelse(ts[["pos"]] %in% c("$.", "$,"), FALSE, TRUE)[2L:nrow(ts)], FALSE)
-  } else {
-    c(grepl("^[.,;!?]$", ts[["word"]])[2L:nrow(ts)], FALSE)
-  }
+  if (verbose) cli_alert_info("decode p-attributes")
+  p_attrs <- if (is.null(p_attributes)) p_attributes(x) else p_attributes
+  ts <- decode(
+    x,
+    to = "data.table",
+    p_attribute = p_attrs,
+    s_attributes = character(),
+    verbose = verbose
+  )
   
-  word_with_whitespace <- paste(ts[["word"]], ifelse(whitespace_after, " ", ""), sep = "")
-  s <- paste(word_with_whitespace, collapse = "")
+  if (!is.null(stoplist)) ts <- ts[!ts[["word"]] %in% stoplist]
   
-  message("... generate token-level annotation")
-  left_offset <- c(1L, (cumsum(nchar(word_with_whitespace)) + 1L)[1L:(nrow(ts) - 1L)])
+  if (verbose) cli_progress_step("generate plain text string")
+  ws_after <- c(!grepl("^[.,;:!?]$", ts[["word"]])[2L:nrow(ts)], FALSE)
+  word_with_ws <- paste(ts[["word"]], ifelse(ws_after, " ", ""), sep = "")
+  s <- stringi::stri_c(word_with_ws, collapse = "")
+  
+  if (verbose) cli_progress_step("generate token-level annotation")
+  left_offset <- c(1L, (cumsum(nchar(word_with_ws)) + 1L)[1L:(nrow(ts) - 1L)])
   right_offset <- left_offset + nchar(ts[["word"]]) - 1L
-  
   w <- NLP::Annotation(
-    id = ranges_to_cpos(from@cpos),
+    id = ts[["cpos"]],
     rep.int("word", nrow(ts)),
     start = left_offset,
     end = right_offset,
-    features = lapply(split(ts[, p_attrs, with = FALSE], f = 1L:nrow(ts)), as.data.frame)
+    features = lapply(
+      split(ts[, p_attrs, with = FALSE], f = 1L:nrow(ts)),
+      as.data.frame
+    )
   )
   
-  message("... which s-attributes are document-level metadata?")
-  s_attrs <- s_attributes(from)
-  meta_candidates <- lapply(
-    setNames(s_attrs, s_attrs),
+  mw_annotations <- lapply(
+    mw,
     function(s_attr){
-      message(sprintf("... checking if s-attribute is metadata: %s ... ", s_attr), appendLF = FALSE)
-      strucs <- cl_cpos2struc(corpus = from@corpus, s_attribute = s_attr, cpos = ranges_to_cpos(from@cpos))
-      if (any(strucs < 0L)){
-        message("NO (not comprehensive)")
-        NULL
-      } else {
-        values <- cl_struc2str(corpus = from@corpus, s_attribute = s_attr, struc = strucs)
-        unique_values <- unique(values)
-        if (length(unique_values) == 1L){
-          message("OK")
-          return(unique_values)
-        } else {
-          message("NO (changes)")
-          character()
-        }
-      }
-    }
-  )
-  meta <- meta_candidates
-  # attributes that do not cover the entire subcorpus and that have different
-  # values are not metadata - discard it
-  for (i in rev(which(sapply(meta_candidates, length) != 1L))) meta[[i]] <- NULL
-  
-  # s-attributes that do not cover entire subcorpus (negative values) are 
-  # annotations of regions of text
-  s_attr_anno <- names(meta_candidates)[sapply(meta_candidates, is.null)]
-  mw_annotations <- do.call(c, lapply(
-    s_attr_anno,
-    function(s_attr){
-      strucs <- unique(
-        cl_cpos2struc(
-          corpus = from@corpus, s_attribute = s_attr,
-          cpos = ranges_to_cpos(from@cpos)
+      if (verbose)
+        cli_progress_step(
+          "generate annotation for s-attribute {.val {s_attr}}"
         )
+      
+      struc_matrix <- RcppCWB::region_matrix_to_struc_matrix(
+        corpus = x@corpus,
+        registry = x@registry_dir,
+        region_matrix = x@cpos,
+        s_attribute = s_attr
       )
-      strucs <- strucs[which(strucs >= 0L)]
-      do.call(c,
-        lapply(
-          strucs,
-          function(struc){
-            cpos <- cl_struc2cpos(corpus = from@corpus, s_attribute = s_attr, struc = struc)
-            NLP::Annotation(
-              id = -1, # assign id later on, if necessary
-              type = s_attr,
-              start = w[which(w$id == min(cpos))]$start,
-              end = w[which(w$id == max(cpos))]$end,
-              features = list(list(constituents = cpos))
+      strucs <- ranges_to_cpos(struc_matrix)
+      strucs_min <- unique(strucs[strucs >= 0L])
+      if (length(strucs_min) == 0L) return(NULL)
+      
+      regions <- get_region_matrix(
+        corpus = x@corpus,
+        s_attribute = s_attr,
+        strucs = strucs_min,
+        registry = x@registry_dir
+      )
+      
+      str <- RcppCWB::cl_struc2str(
+        corpus = x@corpus,
+        registry = x@registry_dir,
+        s_attribute = s_attr,
+        struc = strucs_min
+      )
+      Encoding(str) <- x@encoding
+      str <- as.nativeEnc(x = str, from = x@encoding)
+      
+      data <- lapply(
+        1:nrow(regions),
+        function(i){
+          matching <- which(w$id %in% regions[i,1]:regions[i,2])
+          
+          # If the region covers tokens that have been dropped due to the 
+          # stoplist, no matches - omit these cases.
+          if (length(matching) > 0L){
+            txt <- stri_sub(
+              str = s,
+              from = w[min(matching)]$start,
+              to = w[max(matching)]$end
             )
+            list(
+              kind = str[i],
+              text = txt,
+              constituents = regions[i,1]:regions[i,2],
+              start = w[min(matching)]$start,
+              end = w[max(matching)]$end
+            )
+          } else {
+            NULL
           }
-        )
+        }
+      )
+      
+      nomatch <- sapply(data, length) == 0
+      if (any(nomatch)) data <- data[-which(nomatch)]
+      
+      NLP::Annotation(
+        id = seq_along(data),
+        type = rep.int(
+          if (is.null(names(mw))) s_attr else names(mw)[which(mw == s_attr)],
+          length(data)
+        ),
+        start = unname(unlist(lapply(data, `[[`, "start"))),
+        end = unname(unlist(lapply(data, `[[`, "end"))),
+        features = lapply(data, `[`, c("kind", "text", "constituents"))
       )
     }
-  ))
+  )
   
-  a <- if (length(mw_annotations) > 0L) c(w, mw_annotations) else w
+  if (verbose) cli_progress_step("retrieve metadata")
+  meta <- lapply(
+    setNames(s_attributes, s_attributes),
+    function(s_attr){
+      struc <- cl_cpos2struc(
+        corpus = x@corpus,
+        registry = x@registry_dir,
+        s_attribute = s_attr,
+        cpos = x@cpos[1,1]
+      )
+      value <- cl_struc2str(
+        corpus = x@corpus,
+        registry = x@registry_dir,
+        s_attribute = s_attr,
+        struc = struc
+      )
+      Encoding(value) <- x@encoding
+      as.nativeEnc(x = value, from = x@encoding)
+    }
+  )
   
-  NLP::AnnotatedPlainTextDocument(s = s, a = a, meta = meta)
-})
+  if (verbose) cli_progress_step("instantiate AnnotatedPlainTextDocument")
+  NLP::AnnotatedPlainTextDocument(
+    s = s,
+    a = c(w, do.call(c, mw_annotations)),
+    meta = meta
+  )
+}
 
 
 #' Decode corpus or subcorpus.
 #' 
-#' Decode \code{corpus} or \code{subcorpus} and return class specified by
-#' argument \code{to}.
+#' Decode `corpus` or `subcorpus` and return class specified by argument `to`.
 #'
 #' The primary purpose of the method is type conversion. By obtaining the corpus
-#' or subcorpus in the format specified by the argument \code{to}, the data can
-#' be processed with tools that do not rely on the Corpus Workbench (CWB).
-#' Supported output formats are \code{data.table} (which can be converted to a
-#' \code{data.frame} or \code{tibble} easily) or an \code{Annotation} object as
-#' defined in the package \code{NLP}. Another purpose of decoding the corpus can
-#' be to rework it, and to re-import it into the CWB (e.g. using the
-#' \code{cwbtools}-package).
+#' or subcorpus in the format specified by the argument `to`, the data can be
+#' processed with tools that do not rely on the Corpus Workbench (CWB).
+#' Supported output formats are `data.table` (which can be converted to a
+#' `data.frame` or `tibble` easily) or an `Annotation` object as defined in the
+#' package 'NLP'. Another purpose of decoding the corpus can be to rework it,
+#' and to re-import it into the CWB (e.g. using the 'cwbtools'-package).
 #'
 #' An earlier version of the method included an option to decode a single
-#' s-attribute, which is not supported any more. See the
-#' \code{s_attribute_decode} function of the package RcppCWB.
+#' s-attribute, which is not supported any more. See the `s_attribute_decode()`
+#' function of the package RcppCWB.
 #'
 #' @return The return value will correspond to the class specified by argument
-#'   \code{to}.
+#'   `to`.
 #' 
-#' @param .Object The \code{corpus} or \code{subcorpus} to decode.
+#' @param .Object The `corpus` or `subcorpus` to decode.
 #' @param to The class of the returned object, stated as a length-one
-#'   \code{character} vector.
-#' @param s_attributes The structural attributes to decode. If \code{NULL}
-#'   (default), all structural attributes will be decoded.
-#' @param p_attributes The positional attributes to decode. If \code{NULL}
-#'   (default), all positional attributes will be decoded.
-#' @param decode A \code{logical} value, whether to decode token ids and struc
-#'   ids to character strings. If \code{FALSE}, the values of columns for p- and
-#'   s-attributes will be \code{integer} vectors. If \code{TRUE} (default), the
-#'   respective columns are \code{character} vectors.
+#'   `character` vector.
+#' @param s_attributes The structural attributes to decode. If `NULL` (default),
+#'   all structural attributes will be decoded.
+#' @param p_attributes The positional attributes to decode. If `NULL` (default),
+#'   all positional attributes will be decoded.
+#' @param stoplist A `character` vector with terms to be dropped.
+#' @param decode A `logical` value, whether to decode token ids and struc ids to
+#'   character strings. If `FALSE`, the values of columns for p- and
+#'   s-attributes will be `integer` vectors. If `TRUE` (default), the respective
+#'   columns are `character` vectors.
+#' @param mw A `character` vector with s-attributes with multiword expressions.
+#'   Used only if `to` is 'AnnotatedPlainTextDocument'.
 #' @param ... Further arguments.
-#' @param verbose A \code{logical} value, whether to output progess messages.
+#' @param verbose A `logical` value, whether to output progess messages.
 #' @exportMethod decode
 #' @importFrom RcppCWB get_region_matrix
 #' @seealso To decode a structural attribute, you can use the
@@ -192,28 +264,29 @@ setAs(from = "corpus", to = "AnnotatedPlainTextDocument", def = function(from){
 #' use(pkg = "RcppCWB", corpus = "REUTERS")
 #' 
 #' # Decode corpus as data.table
-#' dt <- decode("GERMAPARLMINI", to = "data.table")
+#' dt <- decode("REUTERS", to = "data.table")
 #' 
 #' # Decode corpus selectively
-#' dt <- decode("GERMAPARLMINI", to = "data.table", p_attributes = "word", s_attributes = "party")
+#' dt <- decode(
+#'   "REUTERS",
+#'   to = "data.table",
+#'   p_attributes = "word",
+#'   s_attributes = "id"
+#' )
 #' 
 #' # Decode a subcorpus
-#' dt <- corpus("GERMAPARLMINI") %>%
-#'   subset(speaker == "Angela Dorothea Merkel") %>%
-#'   decode(s_attributes = c("speaker", "party", "date"), to = "data.table")
-#' 
-#' # Decode subcorpus selectively
-#' corpus("GERMAPARLMINI") %>%
-#'   subset(speaker == "Angela Dorothea Merkel") %>%
-#'   decode(to = "data.table", p_attributes = "word", s_attributes = "party")
+#' dt <- corpus("REUTERS") %>%
+#'   subset(id %in% c("127", "144")) %>%
+#'   decode(s_attributes = "id", to = "data.table")
 #' 
 #' # Decode partition
-#' P <- partition("REUTERS", places = "kuwait", regex = TRUE)
-#' dt <- decode(P)
+#' dt <- partition("REUTERS", places = "kuwait", regex = TRUE) %>%
+#'   decode(to = "data.table")
 #' 
 #' # Previous versions of polmineR offered an option to decode a single
 #' # s-attribute. This is how you could proceed to get a table with metadata.
-#' dt <- decode(P, s_attribute = "id", decode = FALSE)
+#' dt <- partition("REUTERS", places = "kuwait", regex = TRUE) %>% 
+#'   decode(s_attribute = "id", decode = FALSE, to = "data.table")
 #' dt[, "word" := NULL]
 #' dt[,{list(cpos_left = min(.SD[["cpos"]]), cpos_right = max(.SD[["cpos"]]))}, by = "id"]
 #' 
@@ -233,12 +306,22 @@ setAs(from = "corpus", to = "AnnotatedPlainTextDocument", def = function(from){
 #'   words <- s[a[a$type == "word"]]
 #'   sentences <- s[a[a$type == "sentence"]] # does not yet work perfectly for plenary protocols 
 #'   
-#'   doc <- as(p, "AnnotatedPlainTextDocument")
+#'   doc <- decode(p, to = "AnnotatedPlainTextDocument")
 #' }
 #' }
 #' @rdname decode
 #' @importFrom cli cli_progress_step
-setMethod("decode", "corpus", function(.Object, to = c("data.table", "Annotation"), p_attributes = NULL, s_attributes = NULL, decode = TRUE, verbose = TRUE){
+setMethod("decode", "corpus", function(.Object, to = c("data.table", "Annotation", "AnnotatedPlainTextDocument"), p_attributes = NULL, s_attributes = NULL, mw = NULL, stoplist = NULL, decode = TRUE, verbose = TRUE){
+  
+  if (length(to) != 1L)
+    stop("`decode()` - argument 'to' required to have length 1")
+  if (!to %in% c("data.table", "Annotation", "AnnotatedPlainTextDocument")){
+    cli_alert_danger(
+      "Argument 'to' of `decode()` required to be 'data.table' or 'Annotation'."
+    )
+    stop()
+  }
+  
   if (to == "data.table"){
 
     if (is.null(p_attributes)) p_attributes <- p_attributes(.Object)
@@ -268,14 +351,20 @@ setMethod("decode", "corpus", function(.Object, to = c("data.table", "Annotation
         function(s_attr){
           if (verbose)
             cli_progress_step(sprintf("decoding s-attribute: %s", s_attr))
+          
           struc <- cl_cpos2struc(
-            corpus = .Object@corpus, registry = .Object@registry_dir,
-            s_attribute = s_attr, cpos = 0L:max_cpos
+            corpus = .Object@corpus,
+            registry = .Object@registry_dir,
+            s_attribute = s_attr,
+            cpos = 0L:max_cpos
           )
-          if (decode){
+          
+          if (decode && s_attr_has_values(s_attr, x = .Object)){
             str <- cl_struc2str(
-              corpus = .Object@corpus, registry = .Object@registry_dir,
-              s_attribute = s_attr, struc = struc
+              corpus = .Object@corpus,
+              registry = .Object@registry_dir,
+              s_attribute = s_attr,
+              struc = struc
             )
             Encoding(str) <- encoding(.Object)
             return(as.nativeEnc(str, from = encoding(.Object)))
@@ -298,9 +387,17 @@ setMethod("decode", "corpus", function(.Object, to = c("data.table", "Annotation
     
   } else if (to == "Annotation"){
     y <- as(.Object, "Annotation")
-  } else {
-    stop("The 'to'-argument of the decode()-method is required to be either 'data.table' or 'Annotation'.")
+  } else if (to == "AnnotatedPlainTextDocument"){
+    y <- as.AnnotatedPlainTextDocument(
+      x = as(.Object, "subcorpus"), # fn requires cpos slot
+      p_attributes = p_attributes,
+      s_attributes = s_attributes,
+      mw = mw,
+      stoplist = stoplist,
+      verbose = verbose
+    )
   }
+  
   y
 })
 
@@ -317,61 +414,97 @@ setMethod("decode", "character", function(.Object, to = c("data.table", "Annotat
 
 #' @exportMethod decode
 #' @rdname decode
-setMethod("decode", "slice", function(.Object, to = "data.table", s_attributes = NULL, p_attributes = NULL, decode = TRUE, verbose = TRUE){
+setMethod("decode", "slice", function(.Object, to = c("data.table", "Annotation", "AnnotatedPlainTextDocument"), s_attributes = NULL, p_attributes = NULL, mw = NULL, stoplist = NULL, decode = TRUE, verbose = TRUE){
+  
   if (to == "data.table"){
     
     if (is.null(p_attributes)) p_attributes <- p_attributes(.Object)
-    if (!all(p_attributes %in% p_attributes(.Object))) stop("Not all p_attributes provided are available.")
+    if (!all(p_attributes %in% p_attributes(.Object)))
+      stop("Not all p_attributes provided are available.")
     
     y <- data.table(cpos = ranges_to_cpos(.Object@cpos))
+    setkeyv(y, cols = "cpos") # col cpos used repeatedly to  merge s-attributes
 
     for (p_attr in p_attributes){
-      if (verbose) message("... decoding p_attribute ", p_attr)
+      if (verbose) cli_progress_step("decoding p_attribute {.val {p_attr}}")
       y[, (p_attr) := get_token_stream(.Object, p_attribute = p_attr, decode = decode)]
+      if (verbose) cli_progress_done()
+    }
+    
+    if (is.null(s_attributes)){
+      if (verbose)
+        cli_alert_info(
+          "argument `s_attributes` missing - decode all s-attributes"
+          )
+      s_attributes <- s_attributes(.Object)
     }
     
-    if (is.null(s_attributes)) s_attributes <- s_attributes(.Object)
     if (length(s_attributes) > 0L){
-      if (!all(s_attributes %in% s_attributes(.Object))) stop("Not all s_attributes provided are available.")
-      strucs <- RcppCWB::cl_cpos2struc(
-        corpus = .Object@corpus, registry = .Object@registry_dir,
-        s_attribute = s_attributes[1], cpos = .Object@cpos[,1]
-      )
-      
-      s_attr_dt <- data.table(
-        RcppCWB::get_region_matrix(
-          corpus = .Object@corpus, registry = .Object@registry_dir,
-          s_attribute = s_attributes[1], strucs = strucs
+      if (!all(s_attributes %in% s_attributes(.Object)))
+        stop("Not all s_attributes provided are available.")
+
+      for (i in 1L:length(s_attributes)){
+        if (verbose)
+          cli_progress_step("decoding s_attribute {.val  {s_attributes[i]}}")
+        
+        # Tested: strucs, regions and corpus positions will be identical if
+        # i > 2 and current s-attribute is sibling of previous sibling. But
+        # the cost of learning whether s-attributes are siblings consumes the 
+        # performance gain, so we get strucs, regions and cpos anew.
+        struc_matrix <- RcppCWB::region_matrix_to_struc_matrix(
+          corpus = .Object@corpus,
+          s_attribute = s_attributes[i],
+          registry = .Object@registry_dir,
+          region_matrix = .Object@cpos
         )
-      )
-      setnames(s_attr_dt, old = c("V1", "V2"), new = c("cpos_left", "cpos_right"))
-      s_attr_dt[, "struc" := strucs]
-      
-      for (s_attr in s_attributes){
+        strucs <- RcppCWB::ranges_to_cpos(struc_matrix)
+        if (length(strucs) == 0L) next
+        
+        regions <- RcppCWB::get_region_matrix(
+          corpus = .Object@corpus,
+          registry = .Object@registry_dir,
+          s_attribute = s_attributes[i],
+          strucs = strucs
+        )
+        cpos <- RcppCWB::ranges_to_cpos(regions)
+        
+        decode <- decode && s_attr_has_values(s_attributes[i], x = .Object)
         if (decode){
-          if (verbose) message("... decoding s_attribute ", s_attr)
-          str <- cl_struc2str(corpus = .Object@corpus, registry = .Object@registry_dir, s_attribute = s_attr, struc = strucs)
+          str <- cl_struc2str(
+            corpus = .Object@corpus,
+            registry = .Object@registry_dir,
+            s_attribute = s_attributes[i],
+            struc = strucs
+          )
           Encoding(str) <- encoding(.Object)
-          s_attr_dt[, (s_attr) := as.nativeEnc(str, from = encoding(.Object))]
-        } else {
-          s_attr_dt[, (s_attr) := strucs]
         }
+        
+        values <- data.table(
+          cpos = cpos,
+          attr = do.call(
+            c,
+            mapply(
+              rep,
+              x = if (decode) str else strucs,
+              times = (regions[,2] - regions[,1]) + 1,
+              SIMPLIFY = FALSE
+            )
+          )
+        )
+        setnames(values, old = "attr", new = s_attributes[i])
+
+        y <- values[y, on = "cpos"]
+        if (verbose) cli_progress_done()
       }
       
-      unfold <- function(.SD){
-        dt <- data.table(cpos = .SD[["cpos_left"]]:.SD[["cpos_right"]])
-        for (s_attr in s_attributes){
-          value <- .SD[[s_attr]]
-          dt[, (s_attr) := value]
-        }
-        dt
-      }
-      s_attr_dt_ext <- s_attr_dt[, unfold(.SD), by = "struc"]
-      y <- y[s_attr_dt_ext, on = "cpos"]
       setcolorder(y, neworder = c("cpos", p_attributes, s_attributes))
     }
   } else if (to == "Annotation"){
     y <- as(.Object, "Annotation")
+  } else if (to == "AnnotatedPlainTextDocument"){
+    return(callNextMethod())
+  } else {
+    stop("not implemented")
   }
   y
 })
@@ -384,7 +517,21 @@ setMethod("decode", "partition", function(.Object, to = "data.table", s_attribut
 
 #' @exportMethod decode
 #' @rdname decode
-setMethod("decode", "subcorpus", function(.Object, to = "data.table", s_attributes = NULL, p_attributes = NULL, decode = TRUE, verbose = TRUE){
+setMethod("decode", "subcorpus", function(.Object, to = c("data.table", "Annotation", "AnnotatedPlainTextDocument"), s_attributes = NULL, p_attributes = NULL, mw = NULL, stoplist = NULL, decode = TRUE, verbose = TRUE){
+  
+  if (to == "AnnotatedPlainTextDocument"){
+    return(
+      as.AnnotatedPlainTextDocument(
+        x = .Object,
+        p_attributes = p_attributes,
+        s_attributes = s_attributes,
+        mw = mw,
+        stoplist = stoplist,
+        verbose = verbose
+      )
+    )
+  }
+  
   callNextMethod()
 })
 
