@@ -307,13 +307,67 @@ setMethod("s_attributes", "subcorpus", function (.Object, s_attribute = NULL, un
   callNextMethod()
 )
 
+#' @rdname s_attributes-method
+#' @details If `.Object` is a `context` object, the s-attribute value for the
+#'   first corpus position of every match is returned in a character vector.
+#'   If the match is outside a region of the s-attribute, `NA` is returned.
+setMethod("s_attributes", "context", function (.Object, s_attribute = NULL){
+  dt <- slot(.Object, "cpos")
+  dt_min <- dt[dt[["position"]] == 0]
+  setorderv(dt_min, cols = "cpos", order = 1L)
+  cpos <- dt_min[, list(cpos = .SD[["cpos"]][1]), by = "match_id"][["cpos"]]
+  strucs <- cl_cpos2struc(
+    corpus = .Object@corpus,
+    s_attribute = s_attribute,
+    registry = .Object@registry_dir,
+    cpos = cpos
+  )
+  s_attr <- cl_struc2str(
+    corpus = .Object@corpus,
+    s_attribute = s_attribute,
+    struc = strucs
+  )
+  Encoding(s_attr) <- .Object@encoding
+  as.nativeEnc(s_attr, from = .Object@encoding)
+})
+
+
 
 
 #' @docType methods
-#' @rdname partition_bundle-class
-setMethod("s_attributes", "partition_bundle", function(.Object, s_attribute, ...){
-  if ("sAttribute" %in% names(list(...))) s_attribute <- list(...)[["sAttribute"]]
-  lapply(.Object@objects, function(x) s_attributes(x, s_attribute))
+#' @rdname s_attributes-method
+setMethod("s_attributes", "partition_bundle", function(.Object, s_attribute, unique = TRUE, ...){
+
+  if ("sAttribute" %in% names(list(...))){
+    lifecycle::deprecate_warn(
+      when = "0.8.9", 
+      what = "s_attributes(sAttribute)",
+      with = "s_attributes(s_attribute)"
+    )
+    s_attribute <- list(...)[["sAttribute"]]
+  }
+  
+  strucs <- lapply(.Object@objects, slot, "strucs")
+  f <- unlist(
+    mapply(
+      rep,
+      x = seq_along(strucs),
+      times = sapply(strucs, length, USE.NAMES = FALSE)
+    ),
+    recursive = FALSE
+  )
+  values <- cl_struc2str(
+    corpus = .Object@corpus,
+    s_attribute = s_attribute,
+    struc = unlist(strucs, recursive = TRUE),
+    registry = .Object@registry_dir
+  )
+  Encoding(values) <- .Object@encoding
+  values <- as.nativeEnc(values, from = .Object@encoding)
+  retval <- split(x = values, f = f)
+  if (unique) retval <- lapply(retval, unique)
+  names(retval) <- names(.Object@objects)
+  retval
 })
 
 
@@ -355,9 +409,9 @@ setMethod("s_attributes", "call", function(.Object, corpus){
   }
   # for the following recursive function,
   # see http://adv-r.had.co.nz/Expressions.html
-  .fn <- function(x){
+  get_s_attrs <- function(x){
     if (is.call(x)){
-      y <- lapply(x, .fn)
+      y <- lapply(x, get_s_attrs)
     } else if (is.symbol(x)){
       char <- deparse(x)
       if (char %in% s_attrs){
@@ -376,9 +430,34 @@ setMethod("s_attributes", "call", function(.Object, corpus){
     } else {
       y <- NULL
     }
-    unique(unlist(y))
+    unlist(y)
   }
-  .fn(.Object)
+  
+  get_typeof <- function(x){
+    if (is.call(x)){
+      evaluated <- try(eval(x), silent = TRUE)
+      if (is.vector(evaluated)){
+        y <- unique(unlist(lapply(x, get_typeof)))
+      } else {
+        y <- lapply(x, get_typeof)
+      }
+    } else if (is.symbol(x)){
+      y <- NULL
+    } else {
+      y <- typeof(x)
+    }
+    unlist(y)
+  }
+  
+  s_attrs <- get_s_attrs(.Object)
+  types <- get_typeof(.Object)
+  
+  if (length(s_attrs) != length(types)){
+    cli_alert_info("Cannot map s-attributes and types")
+    return(unique(s_attrs))
+  }
+  dt <- unique(data.table(s_attrs, types))
+  setNames(dt[[1]], dt[[2]])
 })
 
 
